@@ -119,21 +119,6 @@ open_request_logger (GPdfURIInput *in, const char *uri, gpointer user_data)
 }
 #endif
 
-static void
-set_window_icon_default (void)
-{
-	GdkPixbuf *pixbuf;
-
-	pixbuf = gdk_pixbuf_new_from_file (GNOMEICONDIR "/gnome-pdf.png", NULL);
-
-	if (pixbuf == NULL)
-		return;
-
-	gtk_window_set_default_icon (pixbuf);
-
-	g_object_unref (pixbuf);
-}
-
 int
 main (int argc, char **argv)
 {
@@ -156,7 +141,7 @@ main (int argc, char **argv)
 	if (bonobo_ui_init ("GNOME PDF Viewer", VERSION, &argc, argv) == FALSE)
 		g_error (_("Could not initialize Bonobo!\n"));
 
-	set_window_icon_default ();
+	gtk_window_set_default_icon_from_file (GNOMEICONDIR "/gnome-pdf.png", NULL);
 
 	client = gnome_master_client ();
 	
@@ -166,7 +151,7 @@ main (int argc, char **argv)
 	ctx = g_value_get_pointer (&value);
 	g_value_unset (&value);
 
-	gtk_idle_add (handle_cmdline_args, ctx);
+	g_idle_add (handle_cmdline_args, ctx);
 
 	uri_in = g_object_new (GPDF_TYPE_URI_INPUT, NULL);
 	recent_facade = g_object_new (GPDF_TYPE_RECENT_FACADE, NULL);
@@ -202,6 +187,8 @@ struct _GPdfWindowPrivate {
 	GtkWidget *slot;
 
 	EggRecentView *recent_view;
+
+	GtkWidget *exit_fullscreen_popup;
 };
 
 /* Drag target types */
@@ -310,8 +297,7 @@ gw_control_load_pdf (GPdfWindow *gpdf_window, const char *uri)
 		control, "IDL:Bonobo/PersistFile:1.0", &ev);
 
 	if (BONOBO_EX (&ev) || persist == CORBA_OBJECT_NIL) {
-		gnome_error_dialog (
-			"Panic: component doesn't implement PersistFile.");
+		g_warning ("Panic: component doesn't implement PersistFile.");
 		goto error_persist;
 	}
 
@@ -348,6 +334,9 @@ gpdf_window_open (GPdfWindow *gpdf_window, const char *name)
 				      GTK_WIDGET (priv->bonobo_widget));
 		gtk_widget_destroy (GTK_WIDGET (priv->bonobo_widget));
 		priv->bonobo_widget = NULL;
+		bonobo_ui_component_set_prop (gpdf_window->priv->ui_component,
+					      "/commands/ViewFullScreen",
+					      "sensitive", "0", NULL);
 		return FALSE;
 	}
 
@@ -355,6 +344,10 @@ gpdf_window_open (GPdfWindow *gpdf_window, const char *name)
 	bonobo_control_frame_control_activate (
 	       bonobo_widget_get_control_frame (priv->bonobo_widget));
 	gtk_widget_show (GTK_WIDGET (priv->bonobo_widget));
+
+	bonobo_ui_component_set_prop (gpdf_window->priv->ui_component,
+				      "/commands/ViewFullScreen",
+				      "sensitive", "1", NULL);
 	
 	return TRUE;
 }
@@ -375,6 +368,11 @@ gpdf_window_close (GPdfWindow *gpdf_window)
 {
 	g_return_if_fail (GPDF_IS_NON_NULL_WINDOW (gpdf_window));
 
+	bonobo_control_frame_set_autoactivate
+	  (bonobo_widget_get_control_frame (gpdf_window->priv->bonobo_widget),
+	   FALSE);
+	bonobo_control_frame_control_deactivate
+	  (bonobo_widget_get_control_frame (gpdf_window->priv->bonobo_widget));
 	gtk_widget_destroy (GTK_WIDGET (gpdf_window));
 
 	if (window_list == NULL)
@@ -389,54 +387,147 @@ gw_close_all (void)
 }
 
 static void
-file_dialog_response (GtkWidget *chooser, gint response, gboolean *dialog_result)
+gpdf_window_set_fullscreen (GPdfWindow *gpdf_window, gboolean fullscreen)
 {
-	gchar *file_name;
-
-	if (response != GTK_RESPONSE_OK) {
-		gtk_widget_hide (chooser);
-		gtk_main_quit ();
-		return;
-	}
-
-	file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
-
-	if (!g_file_test (file_name, G_FILE_TEST_EXISTS)) {
-		g_message ("!file exists in file_dialog_ok: %s", file_name);
-	} else if (g_file_test (file_name, G_FILE_TEST_IS_DIR)) {
-		/* Change into directory if that's what the user selected */
-		gchar *dir_name;
-
-		if (strlen (file_name) < 1) {
-			dir_name = g_strdup ("/");
-		} else {
-			dir_name = g_strdup (file_name);
-		}
-
-		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
-						     dir_name);
-		g_free (dir_name);
-	} else {
-		gtk_widget_hide (chooser);
-		gtk_main_quit ();
-		*dialog_result = TRUE;
-	}
-
-	g_free (file_name);
+	if (fullscreen)
+		gtk_window_fullscreen (GTK_WINDOW (gpdf_window));
+	else
+		gtk_window_unfullscreen (GTK_WINDOW (gpdf_window));
 }
 
-static guint
-file_dialog_delete_event (GtkWidget *widget, GdkEventAny *event)
+static void
+exit_fullscreen_button_clicked_cb (GtkWidget *button, GPdfWindow *window)
 {
-	gtk_widget_hide (widget);
-	gtk_main_quit ();
-	return TRUE;
+	gpdf_window_set_fullscreen (window, FALSE);
+}
+
+static void
+update_exit_fullscreen_popup_position (GPdfWindow *gpdf_window)
+{
+	GdkRectangle screen_rect;
+	int popup_height;
+
+	if (gpdf_window->priv->exit_fullscreen_popup == NULL)
+		return;
+
+	gtk_window_get_size (GTK_WINDOW (gpdf_window->priv->exit_fullscreen_popup),
+                             NULL, &popup_height);
+
+	gdk_screen_get_monitor_geometry (
+		gdk_screen_get_default (),
+		gdk_screen_get_monitor_at_window (gdk_screen_get_default (),
+						  GTK_WIDGET (gpdf_window)->window),
+		&screen_rect);
+
+	gtk_window_move (GTK_WINDOW (gpdf_window->priv->exit_fullscreen_popup),
+			 screen_rect.x, screen_rect.height - popup_height);
+}
+
+static void
+screen_size_changed_cb (GdkScreen *screen, GPdfWindow *gpdf_window)
+{
+	update_exit_fullscreen_popup_position (gpdf_window);
+}
+
+static void
+gpdf_window_init_fullscreen_popup (GPdfWindow *gpdf_window)
+{
+	GtkWidget *popup, *button, *hbox, *icon, *label;
+	GtkAccelGroup *accel_group;
+
+	accel_group = gtk_accel_group_new ();
+	
+	popup = gtk_window_new (GTK_WINDOW_POPUP);
+	gpdf_window->priv->exit_fullscreen_popup = popup;
+
+	button = gtk_button_new ();
+	g_signal_connect (button, "clicked",
+			  G_CALLBACK (exit_fullscreen_button_clicked_cb),
+			  gpdf_window);
+	gtk_widget_add_accelerator (button, "clicked", accel_group,
+				    GDK_Escape, 0,
+				    GTK_ACCEL_VISIBLE);
+	gtk_widget_add_accelerator (button, "clicked", accel_group,
+				    GDK_F11, 0,
+				    GTK_ACCEL_VISIBLE);
+	gtk_widget_show (button);
+	gtk_container_add (GTK_CONTAINER (popup), button);
+
+	hbox = gtk_hbox_new (FALSE, 2);
+	gtk_widget_show (hbox);
+	gtk_container_add (GTK_CONTAINER (button), hbox);
+
+	icon = gtk_image_new_from_stock (GTK_STOCK_QUIT, GTK_ICON_SIZE_BUTTON);
+	gtk_widget_show (icon);
+	gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
+
+	label = gtk_label_new (_("Exit Full Screen"));
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+	update_exit_fullscreen_popup_position (gpdf_window);
+
+	gtk_widget_show (popup);
+
+	g_signal_connect (G_OBJECT (gdk_screen_get_default ()),
+                          "size-changed", G_CALLBACK (screen_size_changed_cb),
+			  gpdf_window);
+
+	gtk_window_add_accel_group (GTK_WINDOW (gpdf_window), accel_group);
+}
+
+static gboolean
+gpdf_window_window_state_changed (GtkWidget *widget, GdkEventWindowState *event,
+				  gpointer user_data)
+{
+	gboolean fullscreen;
+	GPdfWindow *gpdf_window;
+	BonoboUIComponent *uic;
+
+	if (!(event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN))
+		return FALSE;
+
+	g_return_val_if_fail (GPDF_IS_NON_NULL_WINDOW (widget), FALSE);
+
+	gpdf_window = GPDF_WINDOW (widget);
+	fullscreen = event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN;
+
+	uic = gpdf_window->priv->ui_component;
+	if (uic == NULL)
+		return FALSE;
+
+	bonobo_ui_component_set_prop (uic, "/commands/ViewFullScreen", "state",
+				      fullscreen ? "1" : "0", NULL);
+
+	if (fullscreen) {
+		bonobo_ui_component_freeze (uic, NULL);
+		bonobo_ui_component_set_prop (uic,
+					      "/menu", "hidden", "1", NULL);
+		bonobo_ui_component_set_prop (uic,
+					      "/Toolbar", "hidden", "1", NULL);
+		bonobo_ui_component_thaw (uic, NULL);
+
+		gpdf_window_init_fullscreen_popup (gpdf_window);
+	} else {
+		bonobo_ui_component_freeze (uic, NULL);
+		bonobo_ui_component_set_prop (uic,
+					      "/menu", "hidden", "0", NULL);
+		bonobo_ui_component_set_prop (uic,
+					      "/Toolbar", "hidden", "0", NULL);
+		bonobo_ui_component_thaw (uic, NULL);
+
+		gtk_widget_destroy (gpdf_window->priv->exit_fullscreen_popup);
+		gpdf_window->priv->exit_fullscreen_popup = NULL;
+        }
+
+	return FALSE;
 }
 
 static gboolean
 gw_ask_for_uri (GPdfWindow *gpdf_window, gchar ***uri)
 {
 	GtkFileChooser *chooser;
+	GtkFileFilter *pdf_filter, *all_filter;
 	gboolean accepted = FALSE;
 
 	g_return_val_if_fail (GPDF_IS_NON_NULL_WINDOW (gpdf_window), FALSE);
@@ -451,21 +542,19 @@ gw_ask_for_uri (GPdfWindow *gpdf_window, gchar ***uri)
 								 NULL));
 	gtk_file_chooser_set_select_multiple (chooser, TRUE); 
 	gtk_window_set_modal (GTK_WINDOW (chooser), TRUE);
-
-	/* Connect the signals for Ok and Cancel */
-	g_signal_connect (G_OBJECT (chooser), "response",
-			  G_CALLBACK (file_dialog_response), &accepted);
-
+	pdf_filter = gtk_file_filter_new ();
+	gtk_file_filter_set_name (pdf_filter, _("PDF Documents"));
+	gtk_file_filter_add_mime_type (pdf_filter, "application/pdf");
+	gtk_file_chooser_add_filter (chooser, pdf_filter);
+	all_filter = gtk_file_filter_new ();
+	gtk_file_filter_set_name (all_filter, _("All Files"));
+	gtk_file_filter_add_pattern (all_filter, "*");
+	gtk_file_chooser_add_filter (chooser, all_filter);
+	gtk_file_chooser_set_filter (chooser, pdf_filter);
+	
 	gtk_window_set_position (GTK_WINDOW (chooser), GTK_WIN_POS_MOUSE);
 
-	/* Make sure that we quit the main loop if the window is destroyed */
-	g_signal_connect (G_OBJECT (chooser), "delete_event",
-			  G_CALLBACK (file_dialog_delete_event), NULL);
-
-	/* Run the dialog */
-	gtk_widget_show (GTK_WIDGET (chooser));
-	gtk_grab_add (GTK_WIDGET (chooser));
-	gtk_main ();
+	accepted = (gtk_dialog_run (GTK_DIALOG (chooser)) == GTK_RESPONSE_OK);
 
 	if (accepted) {
 		GSList *files, *iter;
@@ -549,6 +638,23 @@ verb_FileExit_cb (BonoboUIComponent *uic, gpointer user_data,
 	gw_close_all ();
 }
 
+static void
+listener_ViewFullScreen (BonoboUIComponent *uic, const char *path,
+                         Bonobo_UIComponent_EventType type, const char *state,
+                         gpointer user_data)
+{
+	g_return_if_fail (GPDF_IS_NON_NULL_WINDOW (user_data));
+
+	if(type != Bonobo_UIComponent_STATE_CHANGED)
+		return;
+
+	if(!state)
+		return;
+
+	gpdf_window_set_fullscreen(GPDF_WINDOW (user_data), atoi (state));
+}
+
+
 /* Brings attention to a window by raising it and giving it focus */
 static void
 raise_and_focus (GtkWidget *widget)
@@ -761,6 +867,9 @@ gw_setup_toplevel_ui (GPdfWindow *gpdf_window)
 	bonobo_ui_util_set_ui (priv->ui_component, 
 			       DATADIR, "gpdf-window-ui.xml", "GPDF",
 			       NULL);
+
+	bonobo_ui_component_add_listener(priv->ui_component, "ViewFullScreen",
+					 listener_ViewFullScreen, gpdf_window);
 }
 
 static void
@@ -816,6 +925,9 @@ GPdfWindow *gpdf_window_construct (GPdfWindow *gpdf_window)
 	gw_setup_recent_menu (gpdf_window, recent_facade, uri_in);
 	gw_setup_recent_toolitem (gpdf_window, recent_facade, uri_in);
 
+	g_signal_connect (G_OBJECT (gpdf_window), "window_state_event",
+			  G_CALLBACK (gpdf_window_window_state_changed), NULL);
+
 	return gpdf_window;
 }
 
